@@ -21,12 +21,15 @@ IOU server module.
 
 import os
 import base64
+import ntpath
+import stat
 import tempfile
 import socket
 import shutil
 
 from gns3server.modules import IModule
 from gns3server.config import Config
+from gns3dms.cloud.rackspace_ctrl import get_provider
 from .iou_device import IOUDevice
 from .iou_error import IOUError
 from .nios.nio_udp import NIO_UDP
@@ -92,6 +95,7 @@ class IOU(IModule):
         self._udp_start_port_range = iou_config.get("udp_start_port_range", 30001)
         self._udp_end_port_range = iou_config.get("udp_end_port_range", 35000)
         self._host = iou_config.get("host", kwargs["host"])
+        self._console_host = iou_config.get("console_host", kwargs["console_host"])
         self._projects_dir = kwargs["projects_dir"]
         self._tempdir = kwargs["temp_dir"]
         self._working_dir = self._projects_dir
@@ -192,6 +196,7 @@ class IOU(IModule):
         self._allocated_udp_ports.clear()
         self.delete_iourc_file()
 
+        self._working_dir = self._projects_dir
         log.info("IOU module has been reset")
 
     @IModule.route("iou.settings")
@@ -298,14 +303,30 @@ class IOU(IModule):
         updated_iou_path = os.path.join(self.images_directory, iou_path)
         if os.path.isfile(updated_iou_path):
             iou_path = updated_iou_path
+        else:
+            if not os.path.exists(self.images_directory):
+                os.mkdir(self.images_directory)
+            cloud_path = request.get("cloud_path", None)
+            if cloud_path is not None:
+                # Download the image from cloud files
+                _, filename = ntpath.split(iou_path)
+                src = '{}/{}'.format(cloud_path, filename)
+                provider = get_provider(self._cloud_settings)
+                log.debug("Downloading file from {} to {}...".format(src, updated_iou_path))
+                provider.download_file(src, updated_iou_path)
+                log.debug("Download of {} complete.".format(src))
+                # Make file executable
+                st = os.stat(updated_iou_path)
+                os.chmod(updated_iou_path, st.st_mode | stat.S_IEXEC)
+                iou_path = updated_iou_path
 
         try:
             iou_instance = IOUDevice(name,
                                      iou_path,
                                      self._working_dir,
-                                     self._host,
                                      iou_id,
                                      console,
+                                     self._console_host,
                                      self._console_start_port_range,
                                      self._console_end_port_range)
 
