@@ -71,6 +71,7 @@ class Qemu(IModule):
         self._udp_start_port_range = qemu_config.get("udp_start_port_range", 40001)
         self._udp_end_port_range = qemu_config.get("udp_end_port_range", 45500)
         self._host = qemu_config.get("host", kwargs["host"])
+        self._console_host = qemu_config.get("console_host", kwargs["console_host"])
         self._projects_dir = kwargs["projects_dir"]
         self._tempdir = kwargs["temp_dir"]
         self._working_dir = self._projects_dir
@@ -123,6 +124,7 @@ class Qemu(IModule):
         self._qemu_instances.clear()
         self._allocated_udp_ports.clear()
 
+        self._working_dir = self._projects_dir
         log.info("QEMU module has been reset")
 
     @IModule.route("qemu.settings")
@@ -214,6 +216,7 @@ class Qemu(IModule):
                                    self._host,
                                    qemu_id,
                                    console,
+                                   self._console_host,
                                    self._console_start_port_range,
                                    self._console_end_port_range)
 
@@ -598,14 +601,14 @@ class Qemu(IModule):
         if sys.platform.startswith("win"):
             return ""
         try:
-            output = subprocess.check_output([qemu_path, "--version"])
-            match = re.search("QEMU emulator version ([0-9a-z\-\.]+)", output.decode("utf-8"))
+            output = subprocess.check_output([qemu_path, "-version"])
+            match = re.search("version\s+([0-9a-z\-\.]+)", output.decode("utf-8"))
             if match:
                 version = match.group(1)
                 return version
             else:
                 raise QemuError("Could not determine the Qemu version for {}".format(qemu_path))
-        except (OSError, subprocess.CalledProcessError) as e:
+        except subprocess.SubprocessError as e:
             raise QemuError("Error while looking for the Qemu version: {}".format(e))
 
     @IModule.route("qemu.qemu_list")
@@ -614,7 +617,6 @@ class Qemu(IModule):
         Gets QEMU binaries list.
 
         Response parameters:
-        - Server address/host
         - List of Qemu binaries
         """
 
@@ -623,26 +625,35 @@ class Qemu(IModule):
         # look for Qemu binaries in the current working directory and $PATH
         if sys.platform.startswith("win"):
             # add specific Windows paths
-            paths.append(os.path.join(os.getcwd(), "qemu"))
+            if hasattr(sys, "frozen"):
+                # add any qemu dir in the same location as gns3server.exe to the list of paths
+                exec_dir = os.path.dirname(os.path.abspath(sys.executable))
+                for f in os.listdir(exec_dir):
+                    if f.lower().startswith("qemu"):
+                        paths.append(os.path.join(exec_dir, f))
+
             if "PROGRAMFILES(X86)" in os.environ and os.path.exists(os.environ["PROGRAMFILES(X86)"]):
                 paths.append(os.path.join(os.environ["PROGRAMFILES(X86)"], "qemu"))
             if "PROGRAMFILES" in os.environ and os.path.exists(os.environ["PROGRAMFILES"]):
                 paths.append(os.path.join(os.environ["PROGRAMFILES"], "qemu"))
         elif sys.platform.startswith("darwin"):
-            # add a specific location on Mac OS X regardless of what's in $PATH
-            paths.append("/usr/local/bin")
+            # add specific locations on Mac OS X regardless of what's in $PATH
+            paths.extend(["/usr/local/bin", "/opt/local/bin"])
+            if hasattr(sys, "frozen"):
+                paths.append(os.path.abspath(os.path.join(os.getcwd(), "../../../qemu/bin/")))
         for path in paths:
             try:
                 for f in os.listdir(path):
-                    if f.startswith("qemu-system") and os.access(os.path.join(path, f), os.X_OK):
+                    if (f.startswith("qemu-system") or f == "qemu" or f == "qemu.exe") and \
+                            os.access(os.path.join(path, f), os.X_OK) and \
+                            os.path.isfile(os.path.join(path, f)):
                         qemu_path = os.path.join(path, f)
                         version = self._get_qemu_version(qemu_path)
                         qemus.append({"path": qemu_path, "version": version})
             except OSError:
                 continue
 
-        response = {"server": self._host,
-                    "qemus": qemus}
+        response = {"qemus": qemus}
         self.send_response(response)
 
     @IModule.route("qemu.echo")
