@@ -23,13 +23,12 @@ import sys
 import os
 import socket
 import shutil
+import subprocess
 
-from pkg_resources import parse_version
 from gns3server.modules import IModule
 from gns3server.config import Config
 from .virtualbox_vm import VirtualBoxVM
 from .virtualbox_error import VirtualBoxError
-from .vboxwrapper_client import VboxWrapperClient
 from .nios.nio_udp import NIO_UDP
 from ..attic import find_unused_port
 
@@ -61,26 +60,34 @@ class VirtualBox(IModule):
 
     def __init__(self, name, *args, **kwargs):
 
-        # get the vboxwrapper location (only non-Windows platforms)
-        if not sys.platform.startswith("win"):
+        # get the vboxmanage location
+        self._vboxmanage_path = None
+        if sys.platform.startswith("win"):
+            if "VBOX_INSTALL_PATH" in os.environ:
+                self._vboxmanage_path = os.path.join(os.environ["VBOX_INSTALL_PATH"], "VBoxManage.exe")
+            elif "VBOX_MSI_INSTALL_PATH" in os.environ:
+                self._vboxmanage_path = os.path.join(os.environ["VBOX_MSI_INSTALL_PATH"], "VBoxManage.exe")
+        elif sys.platform.startswith("darwin"):
+            self._vboxmanage_path = "/Applications/VirtualBox.app/Contents/MacOS/VBoxManage"
+        else:
             config = Config.instance()
             vbox_config = config.get_section_config(name.upper())
-            self._vboxwrapper_path = vbox_config.get("vboxwrapper_path")
-            if not self._vboxwrapper_path or not os.path.isfile(self._vboxwrapper_path):
+            self._vboxmanage_path = vbox_config.get("vboxmanage_path")
+            if not self._vboxmanage_path or not os.path.isfile(self._vboxmanage_path):
                 paths = [os.getcwd()] + os.environ["PATH"].split(os.pathsep)
-                # look for vboxwrapper in the current working directory and $PATH
+                # look for vboxmanage in the current working directory and $PATH
                 for path in paths:
                     try:
-                        if "vboxwrapper" in os.listdir(path) and os.access(os.path.join(path, "vboxwrapper"), os.X_OK):
-                            self._vboxwrapper_path = os.path.join(path, "vboxwrapper")
+                        if "vboxmanage" in [s.lower() for s in os.listdir(path)] and os.access(os.path.join(path, "vboxmanage"), os.X_OK):
+                            self._vboxmanage_path = os.path.join(path, "vboxmanage")
                             break
                     except OSError:
                         continue
 
-            if not self._vboxwrapper_path:
-                log.warning("vboxwrapper couldn't be found!")
-            elif not os.access(self._vboxwrapper_path, os.X_OK):
-                log.warning("vboxwrapper is not executable")
+        if not self._vboxmanage_path:
+            log.warning("vboxmanage couldn't be found!")
+        elif not os.access(self._vboxmanage_path, os.X_OK):
+            log.warning("vboxmanage is not executable")
 
         # a new process start when calling IModule
         IModule.__init__(self, name, *args, **kwargs)
@@ -94,62 +101,10 @@ class VirtualBox(IModule):
         self._udp_start_port_range = vbox_config.get("udp_start_port_range", 35001)
         self._udp_end_port_range = vbox_config.get("udp_end_port_range", 35500)
         self._host = vbox_config.get("host", kwargs["host"])
+        self._console_host = vbox_config.get("console_host", kwargs["console_host"])
         self._projects_dir = kwargs["projects_dir"]
         self._tempdir = kwargs["temp_dir"]
         self._working_dir = self._projects_dir
-        self._vboxmanager = None
-        self._vboxwrapper = None
-
-    def _start_vbox_service(self):
-        """
-        Starts the VirtualBox backend.
-        vboxapi on Windows or vboxwrapper on other platforms.
-        """
-
-        if sys.platform.startswith("win"):
-            import pywintypes
-            import win32com.client
-
-            try:
-                if win32com.client.gencache.is_readonly is True:
-                    # dynamically generate the cache
-                    # http://www.py2exe.org/index.cgi/IncludingTypelibs
-                    # http://www.py2exe.org/index.cgi/UsingEnsureDispatch
-                    win32com.client.gencache.is_readonly = False
-                    #win32com.client.gencache.Rebuild()
-                    win32com.client.gencache.GetGeneratePath()
-
-                win32com.client.gencache.EnsureDispatch("VirtualBox.VirtualBox")
-            except pywintypes.com_error:
-                raise VirtualBoxError("VirtualBox is not installed.")
-
-            try:
-                from .vboxapi_py3 import VirtualBoxManager
-                self._vboxmanager = VirtualBoxManager(None, None)
-                vbox_major_version, vbox_minor_version, _ = self._vboxmanager.vbox.version.split('.')
-                if parse_version("{}.{}".format(vbox_major_version, vbox_minor_version)) <= parse_version("4.1"):
-                    raise VirtualBoxError("VirtualBox version must be >= 4.2")
-            except Exception as e:
-                self._vboxmanager = None
-                raise VirtualBoxError("Could not initialize the VirtualBox Manager: {}".format(e))
-
-            log.info("VirtualBox Manager has successful started: version is {} r{}".format(self._vboxmanager.vbox.version,
-                                                                                           self._vboxmanager.vbox.revision))
-        else:
-
-            if not self._vboxwrapper_path:
-                raise VirtualBoxError("No vboxwrapper path has been configured")
-
-            if not os.path.isfile(self._vboxwrapper_path):
-                raise VirtualBoxError("vboxwrapper path doesn't exist {}".format(self._vboxwrapper_path))
-
-            self._vboxwrapper = VboxWrapperClient(self._vboxwrapper_path, self._tempdir, "127.0.0.1")
-            #self._vboxwrapper.connect()
-            try:
-                self._vboxwrapper.start()
-            except VirtualBoxError:
-                self._vboxwrapper = None
-                raise
 
     def stop(self, signum=None):
         """
@@ -161,10 +116,10 @@ class VirtualBox(IModule):
         # delete all VirtualBox instances
         for vbox_id in self._vbox_instances:
             vbox_instance = self._vbox_instances[vbox_id]
-            vbox_instance.delete()
-
-        if self._vboxwrapper and self._vboxwrapper.started:
-            self._vboxwrapper.stop()
+            try:
+                vbox_instance.delete()
+            except VirtualBoxError:
+                continue
 
         IModule.stop(self, signum)  # this will stop the I/O loop
 
@@ -202,9 +157,7 @@ class VirtualBox(IModule):
         self._vbox_instances.clear()
         self._allocated_udp_ports.clear()
 
-        if self._vboxwrapper and self._vboxwrapper.connected():
-            self._vboxwrapper.send("vboxwrapper reset")
-
+        self._working_dir = self._projects_dir
         log.info("VirtualBox module has been reset")
 
     @IModule.route("virtualbox.settings")
@@ -214,7 +167,7 @@ class VirtualBox(IModule):
 
         Optional request parameters:
         - working_dir (path to a working directory)
-        - vboxwrapper_path (path to vboxwrapper)
+        - vboxmanage_path (path to vboxmanage)
         - project_name
         - console_start_port_range
         - console_end_port_range
@@ -249,10 +202,10 @@ class VirtualBox(IModule):
             self._working_dir = new_working_dir
             for vbox_id in self._vbox_instances:
                 vbox_instance = self._vbox_instances[vbox_id]
-                vbox_instance.working_dir = os.path.join(self._working_dir, "vbox", "vm-{}".format(vbox_instance.id))
+                vbox_instance.working_dir = os.path.join(self._working_dir, "vbox", "{}".format(vbox_instance.name))
 
-        if "vboxwrapper_path" in request:
-            self._vboxwrapper_path = request["vboxwrapper_path"]
+        if "vboxmanage_path" in request:
+            self._vboxmanage_path = request["vboxmanage_path"]
 
         if "console_start_port_range" in request and "console_end_port_range" in request:
             self._console_start_port_range = request["console_start_port_range"]
@@ -272,6 +225,7 @@ class VirtualBox(IModule):
         Mandatory request parameters:
         - name (VirtualBox VM name)
         - vmname (VirtualBox VM name in VirtualBox)
+        - linked_clone (Flag to create a linked clone)
 
         Optional request parameters:
         - console (VirtualBox VM console port)
@@ -290,22 +244,23 @@ class VirtualBox(IModule):
 
         name = request["name"]
         vmname = request["vmname"]
+        linked_clone = request["linked_clone"]
         console = request.get("console")
         vbox_id = request.get("vbox_id")
 
         try:
 
-            if not self._vboxwrapper and not self._vboxmanager:
-                self._start_vbox_service()
+            if not self._vboxmanage_path or not os.path.exists(self._vboxmanage_path):
+                raise VirtualBoxError("Could not find VBoxManage, is VirtualBox correctly installed?")
 
-            vbox_instance = VirtualBoxVM(self._vboxwrapper,
-                                         self._vboxmanager,
+            vbox_instance = VirtualBoxVM(self._vboxmanage_path,
                                          name,
                                          vmname,
+                                         linked_clone,
                                          self._working_dir,
-                                         self._host,
                                          vbox_id,
                                          console,
+                                         self._console_host,
                                          self._console_start_port_range,
                                          self._console_end_port_range)
 
@@ -418,10 +373,7 @@ class VirtualBox(IModule):
         try:
             vbox_instance.start()
         except VirtualBoxError as e:
-            if self._vboxwrapper:
-                self.send_custom_error("{}: {}".format(e, self._vboxwrapper.read_stderr()))
-            else:
-                self.send_custom_error(str(e))
+            self.send_custom_error(str(e))
             return
         self.send_response(True)
 
@@ -759,6 +711,21 @@ class VirtualBox(IModule):
         response = {"port_id": request["port_id"]}
         self.send_response(response)
 
+    def _execute_vboxmanage(self, command):
+        """
+        Executes VBoxManage and return its result.
+
+        :param command: command to execute (list)
+
+        :returns: VBoxManage output
+        """
+
+        try:
+            result = subprocess.check_output(command, stderr=subprocess.STDOUT, timeout=30)
+        except subprocess.SubprocessError as e:
+            raise VirtualBoxError("Could not execute VBoxManage {}".format(e))
+        return result.decode("utf-8")
+
     @IModule.route("virtualbox.vm_list")
     def vm_list(self, request):
         """
@@ -769,26 +736,37 @@ class VirtualBox(IModule):
         - List of VM names
         """
 
-        if not self._vboxwrapper and not self._vboxmanager:
+        try:
+
+            if request and "vboxmanage_path" in request:
+                vboxmanage_path = request["vboxmanage_path"]
+            else:
+                vboxmanage_path = self._vboxmanage_path
+
+            if not vboxmanage_path or not os.path.exists(vboxmanage_path):
+                raise VirtualBoxError("Could not find VBoxManage, is VirtualBox correctly installed?")
+
+            command = [vboxmanage_path, "--nologo", "list", "vms"]
+            result = self._execute_vboxmanage(command)
+        except VirtualBoxError as e:
+            self.send_custom_error(str(e))
+            return
+
+        vms = []
+        for line in result.splitlines():
+            vmname, uuid = line.rsplit(' ', 1)
+            vmname = vmname.strip('"')
+            if vmname == "<inaccessible>":
+                continue  # ignore inaccessible VMs
             try:
-                self._start_vbox_service()
+                extra_data = self._execute_vboxmanage([vboxmanage_path, "getextradata", vmname, "GNS3/Clone"]).strip()
             except VirtualBoxError as e:
                 self.send_custom_error(str(e))
                 return
+            if not extra_data == "Value: yes":
+                vms.append(vmname)
 
-        if self._vboxwrapper:
-            vms = self._vboxwrapper.get_vm_list()
-        elif self._vboxmanager:
-            vms = []
-            machines = self._vboxmanager.getArray(self._vboxmanager.vbox, "machines")
-            for machine in range(len(machines)):
-                vms.append(machines[machine].name)
-        else:
-            self.send_custom_error("Vboxmanager hasn't been initialized!")
-            return
-
-        response = {"server": self._host,
-                    "vms": vms}
+        response = {"vms": vms}
         self.send_response(response)
 
     @IModule.route("virtualbox.echo")
